package com.hisschemoller.controller 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.enum.FileDataType;
	import com.hisschemoller.model.vo.AlertVO;
	
	import org.puremvc.as3.interfaces.ICommand;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.command.SimpleCommand;
	
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.FileFilter;
	import flash.net.FileReference;	
	/**
	 * Opens an MP3 or WAV file from the local file system and creates a 
	 * SoundVO object for the sound. The SoundVO is then stored in the 
	 * SoundBankProxy.
	 * 
	 * @author Wouter Hisschemoller
	 * (c) Oct 2, 2009
	 */
	public class FileOpenCommand extends SimpleCommand implements ICommand
	{
		private var _xml : XML;
		private var _fileReference : FileReference;
		
		override public function execute(notification : INotification) : void
		{
			_xml = XML(notification.getBody());
			
			_fileReference = new FileReference();
			_fileReference.addEventListener(Event.SELECT, fileReferenceSelectHandler, false, 0, true);
			_fileReference.addEventListener(Event.CANCEL, noSelectionHandler, false, 0, true);
			_fileReference.addEventListener(IOErrorEvent.IO_ERROR, selectionErrorHandler, false, 0, true);
			_fileReference.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
			_fileReference.browse([ new FileFilter("MP3 and WAV Files", "*.mp3; *.wav") ]);
		}
		
		/**
		 * 
		 */
		private function noSelectionHandler(inEvent : Event) : void
		{
			removeListeners();
		}
		
		/**
		 * 
		 */
		private function selectionErrorHandler(inEvent : Event) : void
		{
			facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "An error occurred while selecting a file. Please try again."));
			removeListeners();
		}

		/**
		 * @param event : Input / output error event.
		 */
		private function securityErrorHandler(event : SecurityErrorEvent) : void
		{
			facade.sendNotification(AppFacade.ALERT, new AlertVO("Error opening file", "An security error occurred while selecting a file. Please try again."));
		}

		/**
		 * A local file is selected, convert the FileReference.data byteArray to a Sound object
		 */
		private function fileReferenceSelectHandler(event : Event) : void
		{
			removeListeners();
			
			switch(_fileReference.name.substr(_fileReference.name.length - 4, 4).toLowerCase())
			{
				case ".mp3":
				case ".wav":
					_fileReference.addEventListener(Event.COMPLETE, fileReferenceCompleteHandler, false, 0, true);
					_fileReference.load();
					break;
				default:
					facade.sendNotification(AppFacade.ALERT, new AlertVO("Error opening file", "The selected file could not be opened. Only WAV and MP3 files are supported. Please try again."));
					break;
			}
		}

		/**
		 * A WAV file load from the local computer is complete.
		 * @param event : Event.
		 */
		private function fileReferenceCompleteHandler(event : Event) : void
		{
			removeListeners();
			
			facade.sendNotification(AppFacade.FILE_DATA, _fileReference, FileDataType.OPEN_LOCAL_AUDIO_FILE);
		}

		/**
		 * 
		 */
		private function removeListeners() : void
		{
			_fileReference.removeEventListener(Event.SELECT, fileReferenceSelectHandler);
			_fileReference.removeEventListener(Event.CANCEL, noSelectionHandler);
			_fileReference.removeEventListener(IOErrorEvent.IO_ERROR, selectionErrorHandler);
			_fileReference.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
			_fileReference.removeEventListener(Event.COMPLETE, fileReferenceCompleteHandler);
		}
	}
}
