package com.hisschemoller.model 
{
	import de.popforge.audio.output.Audio;

	import com.hisschemoller.model.vo.WaveDisplayInfoVO;
	import com.hisschemoller.utils.audio.vo.SoundVO;

	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;

	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.utils.Timer;

	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplayProxy extends Proxy implements IProxy 
	{
		public static const NAME_SUFFIX : String = '_WaveDisplayProxy';
		public static const GRAPHIC_UPDATED : String = 'GraphicUpdated';
		public static const HORIZONTAL_ZOOM_CHANGED : String = 'horizontalZoomChanged';
		public static const VERTICAL_ZOOM_CHANGED : String = 'verticalZoomChanged';
		public static const CHANNELS_CHANGED : String = 'channelsChanged';
		private var _soundVO : SoundVO;
		private var _updateIntervalTimer : Timer = new Timer(50, 1);
		private var _drawShape : Shape = new Shape();
		private var _minSamplesPerPixel : Number = 1;
		private var _maxSamplesPerPixel : Number = 1;
		private var _currentSamplesPerPixel : Number = 1;
		private var _minVerticalZoomAmount : Number = 1;
		private var _maxVerticalZoomAmount : Number = 10;
		private var _pollingInterval : Number = 1;
		private var _numPolledSamples : int;
		private var _bytesToSkip : int;
		private var _displayWidth : int = 500;
		private var _displayHeight : int = 280;
		private var _displayMiddleL : int;
		private var _displayMiddleR : int;
		private var _displayAmplitudeHeight : int;
		private var _firstSampleInView : int;
		private var _numSamplesInView : Number;
		private var _numSamplesTotal : int;
		private var _verticalZoomAmount : Number;
		private var _filledGraphicTreshold : int = 20;
		/** Variables concerning dragged move and zoom. */
		private var _dragXAtMouseDown : Number;
		private var _previousMouseY : Number;
		private var _zoomCenterPosition : Number;
		private var _sampleAtZoomCenter : int;
		private var _waveDisplayVO : WaveDisplayInfoVO = new WaveDisplayInfoVO();

		public function WaveDisplayProxy(name : String)
		{
			super(name);
		}
		
		/**
		 * Called by the meditor when it's registered.
		 */
		public function mediatorRegistered() : void
		{
			facade.sendNotification(WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED);
			facade.sendNotification(WaveDisplayProxy.VERTICAL_ZOOM_CHANGED, 0);
			
			forceUpdate();
		}

		/**
		 * Change the sound to display.
		 * @param soundVO : The sound to display.
		 */
		public function changeSound(soundVO : SoundVO) : void 
		{
			if(soundVO == _soundVO) return;
			
			if(!soundVO) 
			{
				_waveDisplayVO.bitmapData.fillRect(_waveDisplayVO.bitmapData.rect, 0);
				_currentSamplesPerPixel = 0;
				_firstSampleInView = 0;
				_numSamplesInView = 0;
				_numSamplesTotal = 0;
				sendNotification(WaveDisplayProxy.GRAPHIC_UPDATED, _waveDisplayVO);
				return;
			}
			
			_soundVO = soundVO;
			_waveDisplayVO.bitmapData = new BitmapData(_displayWidth, _displayHeight, true, 0xFF0000);
			
			if(_soundVO.channels == Audio.MONO)
			{
				_displayMiddleL = _displayHeight * 0.5;
				_displayMiddleR = _displayHeight * 0.5;
				_verticalZoomAmount = _displayMiddleL;
				_displayAmplitudeHeight = _displayHeight / 2;
			}
			else if(_soundVO.channels == Audio.STEREO)
			{
				_displayMiddleL = _displayHeight * 0.25;
				_displayMiddleR = _displayHeight * 0.75;
				_verticalZoomAmount = _displayMiddleL;
				_displayAmplitudeHeight = _displayHeight / 4;
			}
			
			_numSamplesTotal = _soundVO.numSamplesTotal;
			_maxSamplesPerPixel = Number(_numSamplesTotal) / Number(_displayWidth);
			_firstSampleInView = 0;
			_zoomCenterPosition = 0.5;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesTotal * _zoomCenterPosition);
			
			currentSamplesPerPixel = _maxSamplesPerPixel;
			verticalZoomLevel = 0;
			
			facade.sendNotification(WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED);
			facade.sendNotification(WaveDisplayProxy.VERTICAL_ZOOM_CHANGED, 0);
			facade.sendNotification(WaveDisplayProxy.CHANNELS_CHANGED, _soundVO.channels);
			
			forceUpdate();
		}

		/**
		 * Resets the state of the proxy.
		 */
		public function clear() : void
		{
			_maxSamplesPerPixel = 1;
			_currentSamplesPerPixel = 1;
			_pollingInterval = 1;
			_firstSampleInView = 0;
			_numSamplesInView = 0;
			_numSamplesTotal = 0;
			_zoomCenterPosition = 0.5;
			_sampleAtZoomCenter = 0;
			_verticalZoomAmount = _displayMiddleL;
			_soundVO = null;
			
			updateGraphic();
		}
		
		/**
		 * 
		 */
		public function forceUpdate() : void
		{
			_updateIntervalTimer.stop();
			updateGraphic();
		}

		/**
		 * @description: The mouse is pressed in the wave display area.
		 * @param xAtMouseDown : mouseX at MOUSE_DOWN
		 * @param yAtMouseDown : mouseY at MOUSE_DOWN
		 */
		public function initializeDrag(xAtMouseDown : Number, yAtMouseDown : Number) : void
		{
			_dragXAtMouseDown = xAtMouseDown;
			_previousMouseY = yAtMouseDown;
			
			_zoomCenterPosition = xAtMouseDown / _displayWidth;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesInView * _zoomCenterPosition);
		}

		/**
		 * @description: The mouse is dragged in the wave display area. In response
		 * 		the waveform should be moved according to the horizontal mouse movement
		 * 		and zoomed in or out following the vertical mouse movement.
		 * @param newDragX : mouseX at MOUSE_MOVE
		 * @param newDragY : mouseY at MOUSE_MOVE
		 */
		public function updateDrag(newDragX : Number, newDragY : Number) : void
		{
			/** The distance the mouse was dragged. */
			var distanceX : Number = newDragX - _dragXAtMouseDown;
			var distanceY : Number = newDragY - _previousMouseY;
			_previousMouseY = newDragY;
			
			/** The new zoom level. */
			_currentSamplesPerPixel -= (distanceY * _currentSamplesPerPixel * 0.01);
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(_currentSamplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _displayWidth;
			
			/** The new position. */
			_zoomCenterPosition = (_dragXAtMouseDown + distanceX) / _displayWidth;
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			/** Polling stuff. */
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_bytesToSkip = Math.round(_pollingInterval - 1) * AudioDriverProxy.BYTES_PER_SAMPLE;
			
			facade.sendNotification(WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED);
			
			updateGraphic();
		}

		/**
		 * @return : horizontal zoom as Number between 0 and 1.
		 */
		public function get horizontalZoomLevel() : Number
		{
			var value : Number = (_currentSamplesPerPixel - _minSamplesPerPixel) / (_maxSamplesPerPixel - _minSamplesPerPixel);
			var valueUnscaled : Number = Math.sqrt(value);
			
			return valueUnscaled;
		}

		/**
		 * @param inValue : Number between 0 and 1.
		 */
		public function set horizontalZoomLevel(value : Number) : void
		{
			/** Scale value for more precision when zoomed in. */
			var valueScaled : Number = Math.pow(value, 2);
			
			currentSamplesPerPixel = _minSamplesPerPixel + ((_maxSamplesPerPixel - _minSamplesPerPixel) * valueScaled);
		}

		/**
		 * Sets the zoom level of the display. Measured in how many samples one horizontal pixel of the display represents.
		 * @param samplesPerPixel : How many samples one horizontal pixel of the display represents.
		 */
		public function set currentSamplesPerPixel(samplesPerPixel : Number) : void
		{
			if(samplesPerPixel == _currentSamplesPerPixel) return;
			
			/** The new zoom level. */
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(samplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _displayWidth;
			
			/** The new position. */
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			/** Polling stuff. */
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_bytesToSkip = Math.round(_pollingInterval - 1) * AudioDriverProxy.BYTES_PER_SAMPLE;
			
			facade.sendNotification(WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED);
			
			updateGraphic();
		}

		/**
		 * 
		 */
		public function get firstSampleInView() : int
		{
			return _firstSampleInView;
		}

		/**
		 * 
		 */
		public function set firstSampleInView(firstSample : int) : void
		{
			setFirstSampleInView(firstSample);
		}

		/**
		 * 
		 */
		public function get numSamplesInView() : int
		{
			return _numSamplesInView;
		}

		/**
		 * @param value : Number between 0 and 1
		 */
		public function set verticalZoomLevel(value : Number) : void
		{
			_verticalZoomAmount = _displayMiddleL * (_minVerticalZoomAmount + ((_maxVerticalZoomAmount - _minVerticalZoomAmount) * value));
			
			facade.sendNotification(WaveDisplayProxy.VERTICAL_ZOOM_CHANGED, value);
			
			updateGraphic();
		}

		/**
		 * Repositions the displayed part of the wave by setting the first visible sample at the left edge of the display.
		 * @param firstSample : The first visible sample at the left edge of the display.
		 * @param doUpdate : True if the display should be updated after the setting was changed.
		 */
		public function setFirstSampleInView(firstSample : int, doUpdate : Boolean = true) : void
		{
			if(firstSample == _firstSampleInView) return;
			
			_firstSampleInView = firstSample;
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			if(doUpdate) updateGraphic();
		}

		/**
		 * Redraws the display.
		 */
		private function updateGraphic() : void
		{
			/** Delay between updates. */
			if(_updateIntervalTimer.running) return;
			_updateIntervalTimer.start();
			
			_drawShape.graphics.clear();
			_drawShape.graphics.lineStyle(1.2, 0x990000);
			_drawShape.graphics.moveTo(0, _displayMiddleL);
			
			_soundVO.byteArray.position = int(_firstSampleInView * AudioDriverProxy.BYTES_PER_SAMPLE);
			
			switch(_soundVO.channels)
			{
				case Audio.MONO:
					updateGraphicMono();
					break;
				case Audio.STEREO:
					updateGraphicStereo();
					break;
			}
		}
		
		/**
		 * 
		 */
		private function updateGraphicMono() : void
		{
			var samplePosition : Number = 0;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel;
			var pixelIndex : uint = 0;
			var positiveWaveHeight : Number = 0.0;
			var negativeWaveHeight : Number = 0.0;
			
			for(var i : int = 0;i < _numPolledSamples; ++i)
			{
				/** Read the left channel. */
				var amplitude : Number = _soundVO.byteArray.readFloat();
				/** Skip the right channel. */
				_soundVO.byteArray.readFloat();
				/** And jump to the next position to poll. */
				_soundVO.byteArray.position = int(_firstSampleInView + samplePosition) * AudioDriverProxy.BYTES_PER_SAMPLE;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitude > 0) positiveWaveHeight = Math.max(positiveWaveHeight, amplitude);
				else negativeWaveHeight = Math.min(negativeWaveHeight, amplitude);
				
				/** If all sample polling for the current pixel is done draw the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					/** Draw: Filled waves or a line if the graphic is zoomed in far enough. */
					if(_currentSamplesPerPixel > _filledGraphicTreshold)
					{
						/** Zoomed out: Draw filled waveform. */
						_drawShape.graphics.moveTo(pixelIndex, _displayMiddleL + Math.max((negativeWaveHeight * _verticalZoomAmount), -_displayAmplitudeHeight));
						_drawShape.graphics.lineTo(pixelIndex, _displayMiddleL + Math.min((positiveWaveHeight * _verticalZoomAmount), _displayAmplitudeHeight));
					}
					else
					{
						/** Zoomed in: Draw waveform as line. */
						if(positiveWaveHeight > (negativeWaveHeight * -1))
						{
							_drawShape.graphics.lineTo(pixelIndex, _displayMiddleL + Math.min((positiveWaveHeight * _verticalZoomAmount), _displayAmplitudeHeight));
						}
						else
						{
							_drawShape.graphics.lineTo(pixelIndex, _displayMiddleL + Math.max((negativeWaveHeight * _verticalZoomAmount), -_displayAmplitudeHeight));
						}
					}
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel;
					positiveWaveHeight = 0;
					negativeWaveHeight = 0;
					pixelIndex ++;
				}
				
				samplePosition += _pollingInterval;
			}
			
			/** Clear and redraw the bitmapData. */
			_waveDisplayVO.bitmapData.fillRect(_waveDisplayVO.bitmapData.rect, 0);
			_waveDisplayVO.bitmapData.draw(_drawShape);
			_waveDisplayVO.currentSamplesPerPixel = _currentSamplesPerPixel;
			_waveDisplayVO.firstSampleInView = _firstSampleInView;
			_waveDisplayVO.numSamplesInView = _numSamplesInView;
			_waveDisplayVO.numSamplesTotal = _numSamplesTotal;
			
			sendNotification(WaveDisplayProxy.GRAPHIC_UPDATED, _waveDisplayVO);
		}
		
		/**
		 * 
		 */
		private function updateGraphicStereo() : void
		{
			var samplePosition : Number = 0;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel;
			var pixelIndex : uint = 0;
			var positiveWaveHeightL : Number = 0.0;
			var negativeWaveHeightL : Number = 0.0;
			var positiveWaveHeightR : Number = 0.0;
			var negativeWaveHeightR : Number = 0.0;
			var leftX : Number = 0;
			var leftY : Number = _displayMiddleL;
			var rightX : Number = 0;
			var rightY : Number = _displayMiddleR;
			
			for(var i : int = 0;i < _numPolledSamples; ++i)
			{
				/** Read the channels. */
				var amplitudeL : Number = _soundVO.byteArray.readFloat();
				var amplitudeR : Number = _soundVO.byteArray.readFloat();
				/** And jump to the next position to poll. */
				_soundVO.byteArray.position = int(_firstSampleInView + samplePosition) * AudioDriverProxy.BYTES_PER_SAMPLE;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitudeL > 0) positiveWaveHeightL = Math.max(positiveWaveHeightL, amplitudeL);
				else negativeWaveHeightL = Math.min(negativeWaveHeightL, amplitudeL);
				if(amplitudeR > 0) positiveWaveHeightR = Math.max(positiveWaveHeightR, amplitudeR);
				else negativeWaveHeightR = Math.min(negativeWaveHeightR, amplitudeR);
				
				/** If all sample polling for the current pixel is done draw the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					/** Draw: Filled waves or a line if the graphic is zoomed in far enough. */
					if(_currentSamplesPerPixel > _filledGraphicTreshold)
					{
						/** Zoomed out: Draw filled waveform. */
						_drawShape.graphics.moveTo(pixelIndex, _displayMiddleL + Math.max((negativeWaveHeightL * _verticalZoomAmount), -_displayAmplitudeHeight));
						_drawShape.graphics.lineTo(pixelIndex, _displayMiddleL + Math.min((positiveWaveHeightL * _verticalZoomAmount), _displayAmplitudeHeight));
						_drawShape.graphics.moveTo(pixelIndex, _displayMiddleR + Math.max((negativeWaveHeightR * _verticalZoomAmount), -_displayAmplitudeHeight));
						_drawShape.graphics.lineTo(pixelIndex, _displayMiddleR + Math.min((positiveWaveHeightR * _verticalZoomAmount), _displayAmplitudeHeight));
					}
					else
					{
						/** Zoomed in: Draw waveform as line. */
						if(positiveWaveHeightL > (negativeWaveHeightL * -1))
						{
							_drawShape.graphics.moveTo(leftX, leftY);
							leftX = pixelIndex;
							leftY = _displayMiddleL + Math.min((positiveWaveHeightL * _verticalZoomAmount), _displayAmplitudeHeight);
							_drawShape.graphics.lineTo(leftX, leftY);
							
							_drawShape.graphics.moveTo(rightX, rightY);
							rightX = pixelIndex;
							rightY = _displayMiddleR + Math.min((positiveWaveHeightR * _verticalZoomAmount), _displayAmplitudeHeight);
							_drawShape.graphics.lineTo(rightX, rightY);
						}
						else
						{
							_drawShape.graphics.moveTo(leftX, leftY);
							leftX = pixelIndex;
							leftY = _displayMiddleL + Math.max((negativeWaveHeightL * _verticalZoomAmount), -_displayAmplitudeHeight);
							_drawShape.graphics.lineTo(leftX, leftY);
							
							_drawShape.graphics.moveTo(rightX, rightY);
							rightX = pixelIndex;
							rightY = _displayMiddleR + Math.max((negativeWaveHeightR * _verticalZoomAmount), -_displayAmplitudeHeight);
							_drawShape.graphics.lineTo(rightX, rightY);
						}
					}
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel;
					positiveWaveHeightL = 0;
					negativeWaveHeightL = 0;
					positiveWaveHeightR = 0;
					negativeWaveHeightR = 0;
					pixelIndex ++;
				}
				
				samplePosition += _pollingInterval;
			}
			
			/** Clear and redraw the bitmapData. */
			_waveDisplayVO.bitmapData.fillRect(_waveDisplayVO.bitmapData.rect, 0);
			_waveDisplayVO.bitmapData.draw(_drawShape);
			_waveDisplayVO.currentSamplesPerPixel = _currentSamplesPerPixel;
			_waveDisplayVO.firstSampleInView = _firstSampleInView;
			_waveDisplayVO.numSamplesInView = _numSamplesInView;
			_waveDisplayVO.numSamplesTotal = _numSamplesTotal;
			
			sendNotification(WaveDisplayProxy.GRAPHIC_UPDATED, _waveDisplayVO);
		}
	}
}
