package com.hisschemoller.utils.delegates 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.delegates.vo.AudioLoaderVO;
	
	import org.puremvc.as3.interfaces.IFacade;
	
	import mx.rpc.Responder;
	
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLRequest;	
	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 26, 2010
	 */
	public class AudioLoaderDelegate 
	{
		private var _responder : Responder;
		private var _facade : IFacade;
		private var _waitingStack : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>();
		private var _loadingStack : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>();
		private var _loadedStack : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>();
		private var _loaderCount : uint = 2; /** The maximum number of simultaneous loading audio files. */
		
		public function AudioLoaderDelegate(responder : Responder, facade : IFacade)
		{
			_responder = responder;
			_facade = facade;
		}

		/**
		 * Load a series of audio files.
		 * @param fileUrls: A list of audio file URLs.
		 */
		public function loadAudioFiles(fileUrls : Vector.<String>) : void
		{
			for(var i : uint = 0;i < fileUrls.length; i++)
			{
				var url : String = fileUrls[i];
				
				/** If the URL is not valid continue. */
				if(url == null || url == "") continue;
					
				/** If the file's not an MP3 or WAV skip it. */
				var fileExtention : String = url.substr(url.length - 3).toLowerCase();
				if(fileExtention != "wav" && fileExtention != "mp3") continue;
				
				/** If the URL is already in the list continue (double entry). */
				for(var j : uint = 0;j < fileUrls.length; j++)
				{
					if(url == fileUrls[j]) continue;
				}
				
				var audioLoaderVO : AudioLoaderVO = new AudioLoaderVO(url);
				audioLoaderVO.fileExtention = fileExtention;
				
				_waitingStack.push(audioLoaderVO);
				
				loadNextAudioFile();
			}
		}

		/**
		 * Load next sound if the waiting stack is not empty.
		 * @sends SoundLoaderEvent#ALL_LOADED
		 */
		private function loadNextAudioFile() : void 
		{
			/** Quit if all loaders taken. */
			if(_loadingStack.length == _loaderCount) return;

			/** Quit if no waiting data. */
			if(_waitingStack.length == 0) 
			{
				if(_loadingStack.length == 0)
				{
					/** All done. */
					_responder.result(_loadedStack);
				}
				return;
			}
			
			/** Get next object to load. */
			var audioLoadData : AudioLoaderVO = _waitingStack.shift();
			
			/** Store object in loading list. */
			_loadingStack.push(audioLoadData);
			
			/** Create listeners. */
			audioLoadData.urlLoader.dataFormat = URLLoaderDataFormat.BINARY;
			audioLoadData.urlLoader.addEventListener(Event.COMPLETE, urlLoaderEventHandler);
			audioLoadData.urlLoader.addEventListener(Event.OPEN, urlLoaderStartedHandler);
			audioLoadData.urlLoader.addEventListener(ProgressEvent.PROGRESS, urlLoaderProgressEventHandler);
			audioLoadData.urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, urlLoaderEventHandler);
			audioLoadData.urlLoader.addEventListener(IOErrorEvent.IO_ERROR, urlLoaderEventHandler);
			audioLoadData.urlLoader.load(new URLRequest(audioLoadData.url));
		}

		/**
		 * A sound file started download.
		 */
		private function urlLoaderStartedHandler(event : Event) : void 
		{
			var audioLoaderVO : AudioLoaderVO = getAudioDataByLoader(URLLoader(event.target), _loadingStack);
			
			if(audioLoaderVO == null) 
			{
				_facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "No data found for external loaded MP3 file. The audio file will not be loaded."));
				removeEventListeners(audioLoaderVO);
				loadNextAudioFile();
			}
			else
			{
				audioLoaderVO.isLoading = true;
			}
		}

		/**
		 * The URLLoader dispatched an event while loading an audio file.
		 */
		private function urlLoaderEventHandler(event : Event) : void 
		{
			var audioLoaderVO : AudioLoaderVO = getAudioDataByLoader(URLLoader(event.target), _loadingStack);
			
			/** Remove listeners. */
			removeEventListeners(audioLoaderVO);
			
			if(audioLoaderVO == null) 
			{
				_facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "Data for the sound loader not found. An audio file will not be loaded."));
			}
			else if(event is ErrorEvent) 
			{
				_facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "An audio file could not be loaded. Error message: " + ErrorEvent(event).text));
			} 
			else 
			{
				/** Success. */
				audioLoaderVO.byteArray = URLLoader(event.target).data;
				
				/** Transfer data from loadingStack to loadedStack. */
				_loadedStack.push(_loadingStack.splice(_loadingStack.indexOf(audioLoaderVO), 1)[0]);
			}
			
			loadNextAudioFile();
		}

		/**
		 * Handle ProgressEvent from loading audio file.
		 * @param event: ProgressEvent
		 */
		private function urlLoaderProgressEventHandler(event : ProgressEvent) : void 
		{
			var audioLoadData : AudioLoaderVO = getAudioDataByLoader(URLLoader(event.target), _loadingStack);
			
			if(audioLoadData == null) 
			{
				_facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "Data for sound loader not found."));
				return;
			}
			
			/** Load progress code to be inserted here. */
		}

		/**
		 * Get sound data for specified sound
		 */
		private function getAudioDataByLoader(loader : Object, stack : Vector.<AudioLoaderVO>) : AudioLoaderVO 
		{
			for each (var audioLoaderVO : AudioLoaderVO in stack)
			{
				if(loader is URLLoader && audioLoaderVO.urlLoader == loader) return audioLoaderVO;
			}

			return null;
		}
		
		/**
		 * 
		 */
		private function removeEventListeners(audioLoaderVO : AudioLoaderVO) : void
		{
			audioLoaderVO.isLoading = false;
			
			audioLoaderVO.urlLoader.removeEventListener(Event.COMPLETE, urlLoaderEventHandler);
			audioLoaderVO.urlLoader.removeEventListener(Event.OPEN, urlLoaderStartedHandler);
			audioLoaderVO.urlLoader.removeEventListener(ProgressEvent.PROGRESS, urlLoaderProgressEventHandler);
			audioLoaderVO.urlLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, urlLoaderEventHandler);
			audioLoaderVO.urlLoader.removeEventListener(IOErrorEvent.IO_ERROR, urlLoaderEventHandler);
		}
	}
}
