package com.hisschemoller.utils.midi 
{
	import flash.events.Event;

	/**
	 * @author Wouter Hisschemöller
	 * (c) May 12, 2009
	 */
	public class MidiFileVO
	{
		public var format : uint;
		public var trackCount : uint;
		public var ticksPerBeat : uint;
		public var timestampType : String;
		public var events : Vector.<MidiEventVO> = new Vector.<MidiEventVO>();
		// and now for some metadata
		public var usedMidiChannels : Vector.<Boolean> = new Vector.<Boolean>(MidiSpecification.NUM_CHANNELS, true);
		public var error : String;
		
		public function MidiFileVO()
		{
			for(var i : uint = 0;i < MidiSpecification.NUM_CHANNELS; i++)
			{
				usedMidiChannels[i] = false;
			}
		}

		public function parseXML(o : XML) : Boolean
		{
			format = o.Format;
			trackCount = o.TrackCount;
			ticksPerBeat = o.TicksPerBeat;
			timestampType = o.TimestampType;
			
			// only type 0 MIDI format supported
			if(format != 0)
			{
				error = MidiManagerEvent.FORMAT_ERROR;
				return false;
			}
			
			// only timestampType delta (relative) is supported
			if(timestampType != MidiTimestampType.DELTA)
			{
				error = MidiManagerEvent.TIMESTAMP_ERROR;
				return false;
			}
			
			var eventData : XMLList = o..Event;
			var numEvents : uint = eventData.length();
			var eventVO : MidiEventVO;
			
			for(var i : uint = 0;i < numEvents; i++)
			{
				switch(String(eventData[i].*[1].name()))
				{
					case MidiEventType.NOTE_ON:
						eventVO = new MidiNoteOnVO();
						break;
					case MidiEventType.CONTROL_CHANGE:
						eventVO = new MidiControlChangeVO();
						break;
					case MidiEventType.PROGRAM_CHANGE:
						eventVO = new MidiProgramChangeVO();
						break;
					case MidiEventType.SYS_EX:
						eventVO = new MidiSysExVO();
						break;
				}
				
				// if the event was recognized
				if(eventVO != null)
				{
					var success : Boolean = eventVO.parseXML(eventData[i]);
					
					if(!success)
					{
						error = MidiManagerEvent.PARSE_ERROR;
						return false;
					}
					
					events.push(eventVO);
					
					// set a flag indicating the event's channel is in use
					if((eventVO.channel > 0) && (eventVO.channel <= MidiSpecification.NUM_CHANNELS))
					{
						usedMidiChannels[eventVO.channel - 1] = true;
					}
				}
			}
			
			return true;
		}
	}
}
