package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.enum.FileDataType;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.midi.MidiNoteOffVO;
	import com.hisschemoller.utils.midi.MidiNoteOnVO;
	import com.hisschemoller.view.components.WaveControls;
	
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.patterns.mediator.Mediator;
	
	import flash.events.Event;	
	/**
	 * @author Wouter Hisschemoller
	 * (c) Jan 5, 2010
	 */
	public class WaveControlsMediator extends Mediator implements IMediator 
	{
		public static const NAME_SUFFIX : String = '_WaveControlsMediator';
		private var _editorProxy : WaveEditorProxy;
		private var _aboutText : String;

		public function WaveControlsMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent); 
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_editorProxy = facade.retrieveProxy(getMediatorName().substr(0, getMediatorName().length - NAME_SUFFIX.length)) as WaveEditorProxy;
			
			_aboutText = "Audio Editor v1.0. Designed and programmed by Wouter Hisschemöller. ";
			_aboutText += "Visit <a href='http://www.hisschemoller.com'><u>www.hisschemoller.com</u></a> to download the source code.\n\n";
			_aboutText += "Code by others I used: " +
				"<a href='http://code.google.com/p/minimalcomps/'><u>MinimalComps</u></a>, " +
				"<a href='http://www.flexiblefactory.co.uk/flexible/?p=46'><u>MP3FileReferenceLoaderLib</u></a>, " +
				"<a href='http://code.google.com/p/popforge'><u>PopForge</u></a>, " +
				"<a href='http://puremvc.org'><u>PureMVC</u></a>, " +
				"<a href='http://www.nesium.com/products/trazzle/'><u>Trazzle</u></a>, " +
				"<a href='http://www.greensock.com/tweenlite/'><u>TweenLite</u></a>.";
			
			controls.addEventListener(WaveControls.PLAY_PRESSED, buttonEventHandler);
			controls.addEventListener(WaveControls.PLAY_RELEASED, buttonEventHandler);
			controls.addEventListener(WaveControls.OPEN_BUTTON_CLICK, buttonEventHandler);
			controls.addEventListener(WaveControls.SAVE_BUTTON_CLICK, buttonEventHandler);
			controls.addEventListener(WaveControls.ABOUT_BUTTON_CLICK, buttonEventHandler);
		}

		/**
		 * 
		 */
		private function buttonEventHandler(event : Event) : void
		{
			switch(event.type)
			{
				case WaveControls.PLAY_PRESSED:
					var noteOn : MidiNoteOnVO = new MidiNoteOnVO();
					noteOn.channel = 0;
					noteOn.deltaTime = 0;
					noteOn.pitch = 60;
					noteOn.velocity = 100;
					_editorProxy.noteOn(noteOn);
					break;
					
				case WaveControls.PLAY_RELEASED:
					var noteOff : MidiNoteOffVO = new MidiNoteOffVO();
					noteOff.channel = 0;
					noteOff.deltaTime = 0;
					noteOff.pitch = 60;
					noteOff.velocity = 0;
					_editorProxy.noteOff(noteOff);
					break;
					
				case WaveControls.OPEN_BUTTON_CLICK:
					var xml : XML = 
						<instrument id="" type="pitchedsample">
							<programs>
								<program id="" index="" type="pitchedsample" soundid="" start="" end="" reverse="false" transpose="0" tune="0" />
							</programs>
						</instrument>;
					xml.@id = _editorProxy.getProxyName();
					
					facade.sendNotification(AppFacade.FILE_DATA, xml, FileDataType.OPEN_LOCAL_AUDIO_FILE);
					break;
					
				case WaveControls.SAVE_BUTTON_CLICK:
					_editorProxy.saveAudioFile();
					break;
					
				case WaveControls.ABOUT_BUTTON_CLICK:
					facade.sendNotification(AppFacade.ALERT, new AlertVO("about", _aboutText));
					break;
			}
		}

		/**
		 * 
		 */
		private function get controls() : WaveControls
		{
			return viewComponent as WaveControls;
		}
	}
}
