package com.hisschemoller.view.components 
{
	import assets.WaveDisplayBackground;
	import assets.PlayBar;
	import assets.WaveDisplayAsset;

	import de.popforge.audio.output.Audio;

	import com.bit101.components.Component;
	import com.bit101.components.Slider;
	import com.hisschemoller.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.text.BodyText;
	import com.hisschemoller.view.components.ui.BasicScrollBar;
	import com.hisschemoller.view.components.ui.BasicSlider;

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormatAlign;
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplay extends Component 
	{
		public static const HORIZONTAL_ZOOM_CHANGE : String = "horizontalZoomChange";
		public static const VERTICAL_ZOOM_CHANGE : String = "verticalZoomChange";
		public static const POSITION_CHANGE : String = "positionChange";
		public static const MOUSE_DOWN : String = "displayMouseDown";
		public static const MOUSE_DRAG : String = "displayMouseDrag";
		private var _background : WaveDisplayBackground;
		private var _display : WaveDisplayAsset;
		private var _playBar : PlayBar;
		private var _hZoomSlider : BasicSlider;
		private var _vZoomSlider : BasicSlider;
		private var _firstLabel : BodyText;
		private var _lastLabel : BodyText;
		private var _samplesPerPixelLabel : BodyText;
		private var _positionScrollBar : BasicScrollBar;
		private var _waveBitmap : Bitmap;
		private var _sampleProcessor : IAudioProcessor;
		private var _mouseIsDown : Boolean;
		private var _firstSampleInView : int = -1;
		private var _numSamplesInView : int = -1;
		private var _numSamplesTotal : int = -1;
		private var _currentSamplesPerPixel : int = -1;

		public function WaveDisplay(container : DisplayObjectContainer)
		{
			super(container);
		}

		/**
		 * 
		 */
		public function updateGraphic(bitmapData : BitmapData) : void
		{
			_waveBitmap.bitmapData = bitmapData;
		}

		/**
		 * 
		 */
		public function updateHZoomSlider(value : Number) : void
		{
			_hZoomSlider.value = value;
		}

		/**
		 * 
		 */
		public function updateVZoomSlider(value : Number) : void
		{
			_vZoomSlider.value = value;
		}

		/**
		 * 
		 */
		public function updateChannels(value : uint) : void
		{
			switch(value)
			{
				case Audio.MONO:
					_display.tMono.visible = true;
					_display.tStereo.visible = false;
					break;
					
				case Audio.STEREO:
					_display.tMono.visible = false;
					_display.tStereo.visible = true;
					break;
			}
		}

		/**
		 * The waveform was dragged or zoomed. The position scrollbar must be updated.
		 */
		public function updatePositionScrollBar(numSamplesInView : int, numSamplesTotal : int, firstSampleInView : int, currentSamplesPerPixel : Number) : void
		{
			_firstSampleInView = firstSampleInView;
			_numSamplesInView = numSamplesInView;
			_numSamplesTotal = numSamplesTotal;
			_currentSamplesPerPixel = currentSamplesPerPixel;
			
			var min : Number = 0;
			var max : Number = numSamplesTotal - numSamplesInView;
			var value : Number = firstSampleInView;
			_positionScrollBar.setSliderParams(min, max, value);
			_positionScrollBar.setThumbPercent(numSamplesInView / numSamplesTotal);
			_positionScrollBar.setPageSize(numSamplesInView);
			
			_firstLabel.text = firstSampleInView.toString();
			_lastLabel.text = (firstSampleInView + numSamplesInView).toString();
			_samplesPerPixelLabel.text = currentSamplesPerPixel.toFixed(2);
		}

		/**
		 * An IAudioProcessor just started. It should be tracked if it plays the selected program.
		 */
		public function processorAdded(audioProcessor : IAudioProcessor) : void
		{
			_sampleProcessor = audioProcessor;
		}

		/**
		 * An IAudioProcessor just was removed. It should be tracked if it played the selected program.
		 */
		public function processorRemoved() : void
		{
			_sampleProcessor = null;
		}

		/**
		 * Update the play bar every frame.
		 */
		public function enterFrame(streamInfo : AudioStreamInfoVO) : void
		{
			if(_sampleProcessor)
			{
				var sampleIndex : int = _sampleProcessor.sampleIndex + streamInfo.samplesSinceLastBuffer - streamInfo.latencyInSamples;
				
				if(sampleIndex > _firstSampleInView && sampleIndex < _firstSampleInView + _numSamplesInView)
				{
					_playBar.visible = true;
					_playBar.x = _waveBitmap.x + ((sampleIndex - _firstSampleInView) / _currentSamplesPerPixel);
				}
				else
				{
					_playBar.visible = false;
				}
			}
			else
			{
				_playBar.visible = false;
			}
		}

		/**
		 * 
		 */
		public function get hZoomSliderValue() : Number
		{
			return _hZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get vZoomSliderValue() : Number
		{
			return _vZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get positionScrollBarPosition() : Number
		{
			return _positionScrollBar.value;
		}

		/**
		 * 
		 */
		public function get graphicMouseX() : Number
		{
			return _waveBitmap.mouseX;
		}

		/**
		 * 
		 */
		public function get graphicMouseY() : Number
		{
			return _waveBitmap.mouseY;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_background = new WaveDisplayBackground();
			_background.x = 10;
			_background.y = 173;
			_background.doubleClickEnabled = true;
			_background.addEventListener(MouseEvent.MOUSE_DOWN, graphicMouseDownHandler);
			_background.addEventListener(MouseEvent.DOUBLE_CLICK, graphicDoubleClickHandler);
			addChild(_background);
			
			_waveBitmap = new Bitmap();
			_waveBitmap.x = _background.x;
			_waveBitmap.y = _background.y;
			addChild(_waveBitmap);
			
			_display = new WaveDisplayAsset();
			addChild(_display);
			
			_playBar = new PlayBar();
			_playBar.visible = false;
			_playBar.x = 10;
			_playBar.y = 173;
			addChild(_playBar);
			
			_hZoomSlider = new BasicSlider(Slider.HORIZONTAL, this, 110, 480, handleHZoomSliderChange);
			_hZoomSlider.setSliderParams(0, 1, 0);
			_hZoomSlider.setSize(150, 11);
			
			_vZoomSlider = new BasicSlider(Slider.HORIZONTAL, this, 360, 480, handleVZoomSliderChange);
			_vZoomSlider.setSliderParams(0, 1, 0);
			_vZoomSlider.setSize(150, 11);
			
			_firstLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_firstLabel.x = 15;
			_firstLabel.y = 437;
			addChild(_firstLabel);
			
			_lastLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_lastLabel.align = TextFormatAlign.RIGHT;
			_lastLabel.autoSize = TextFieldAutoSize.RIGHT;
			_lastLabel.x = 502;
			_lastLabel.y = 437;
			addChild(_lastLabel);
			
			_samplesPerPixelLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_samplesPerPixelLabel.align = TextFormatAlign.CENTER;
			_samplesPerPixelLabel.autoSize = TextFieldAutoSize.CENTER;
			_samplesPerPixelLabel.x = 260;
			_samplesPerPixelLabel.y = 437;
			addChild(_samplesPerPixelLabel);
			
			_positionScrollBar = new BasicScrollBar(Slider.HORIZONTAL, this, 10, 457, handlePositionScrollBarChange);
			_positionScrollBar.setSize(500, 19);
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.HORIZONTAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.VERTICAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handlePositionScrollBarChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
		}

		/**
		 * 
		 */
		private function graphicMouseDownHandler(event : MouseEvent) : void
		{
			_mouseIsDown = true;
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, graphicMouseMoveHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, handleGraphicMouseUp);
			stage.addEventListener(Event.MOUSE_LEAVE, handleGraphicMouseUp);
			
			dispatchEvent(new Event(WaveDisplay.MOUSE_DOWN));
		}

		/**
		 * 
		 */
		private function graphicMouseMoveHandler(event : MouseEvent) : void
		{
			if(_mouseIsDown)
			{
				dispatchEvent(new Event(WaveDisplay.MOUSE_DRAG));
			}
		}

		/**
		 * 
		 */
		private function handleGraphicMouseUp(event : Event) : void
		{
			_mouseIsDown = false;
			
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, graphicMouseMoveHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, handleGraphicMouseUp);
			stage.removeEventListener(Event.MOUSE_LEAVE, handleGraphicMouseUp);
		}

		/**
		 * 
		 */
		private function graphicDoubleClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(event);
		}
	}
}
