package com.hisschemoller.view.components 
{
	import com.hisschemoller.view.components.ui.BasicScrollBar;
	import assets.ActivityLED;
	import assets.SampleEditorAsset;

	import com.bit101.components.Component;
	import com.bit101.components.Slider;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.text.BodyText;
	import com.hisschemoller.view.components.ui.BasicLocatorBar;
	import com.hisschemoller.view.components.ui.BasicSlider;
	import com.hisschemoller.view.components.ui.PlayLocatorBar;
	import com.hisschemoller.view.components.ui.TextButton;

	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormatAlign;
	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveEditor extends Component 
	{
		public static const START_PLAY_LOCATOR_CHANGE : String = "startPlayLocatorChange";
		public static const END_PLAY_LOCATOR_CHANGE : String = "endPlayLocatorChange";
		public static const REVERSE_CHANGE : String = "reverseChange";
		public static const TRANSPOSE_CHANGE : String = "transposeChange";
		public static const TUNE_CHANGE : String = "tuneChange";
		private var _waveControls : WaveControls;
		private var _waveDisplay : WaveDisplay;
		private var _playLocatorBar : PlayLocatorBar;
		private var _reverseButton : TextButton;
		private var _selectionLabel : BodyText;
		private var _transposeSlider : BasicSlider;
		private var _transposeLabel : BodyText;
		private var _tuneSlider : BasicSlider;
		private var _tuneLabel : BodyText;
		private var _activityLED : ActivityLED;
		private var _startSample : int = -1;
		private var _endSample : int = -1;
		private var _firstSampleInView : int = -1;
		private var _numSamplesInView : int = -1;
		
		public function WaveEditor(parent : DisplayObjectContainer)
		{
			super(parent);
		}
		
		/**
		 * 
		 */
		public function startLocatorChanged(startSample : int) : void
		{
			_startSample = startSample;
			
			updateLocators();
		}
		
		/**
		 * 
		 */
		public function endLocatorChanged(endSample : int) : void
		{
			_endSample = endSample;
			
			updateLocators();
		}
		
		/**
		 * 
		 */
		public function graphicsUpdated(firstSampleInView : int, numSamplesInView : int) : void
		{
			_firstSampleInView = firstSampleInView;
			_numSamplesInView = numSamplesInView;
			
			updateLocators();
		}

		/**
		 * 
		 */
		public function updatePlaybackState(isPlaying : Boolean) : void
		{
			if(isPlaying) _activityLED.gotoAndPlay("play");
			else _activityLED.gotoAndPlay("neutral");
		}

		/**
		 * Return locator position
		 */
		public function getLocatorValue(locator : String) : Number
		{
			switch(locator)
			{
				case START_PLAY_LOCATOR_CHANGE:
					return _playLocatorBar.startValue;
					break;
				case END_PLAY_LOCATOR_CHANGE:
					return _playLocatorBar.endValue;
					break;
			}
			
			return NaN;
		}
		
		/**
		 * Reference to WaveControls required by WaveControlsMediator.
		 */
		public function get waveControls() : Sprite
		{
			return _waveControls;
		}
		
		/**
		 * Reference to WaveDisplay required by WaveDisplayMediator.
		 */
		public function get waveDisplay() : WaveDisplay
		{
			return _waveDisplay;
		}
		
		/**
		 * 
		 */
		public function get transpose() : Number
		{
			return _transposeSlider.value;
		}
		
		/**
		 * 
		 */
		public function set transpose(value : Number) : void
		{
			_transposeSlider.value = value;
			_transposeLabel.text = value.toFixed(2);
		}
		
		/**
		 * 
		 */
		public function get tune() : Number
		{
			return _tuneSlider.value;
		}
		
		/**
		 * 
		 */
		public function set tune(value : Number) : void
		{
			_tuneSlider.value = value;
			_tuneLabel.text = value.toFixed(2);
		}

		/**
		 * 
		 */
		public function set reverse(value : Boolean) : void
		{
			_reverseButton.selected = value;
		}

		/**
		 * 
		 */
		public function get reverse() : Boolean
		{
			return _reverseButton.selected;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			var asset : SampleEditorAsset = new SampleEditorAsset();
			addChild(asset);
			
			var displayContainer : Sprite = new Sprite();
			asset.addChildAt(displayContainer, 0);
			
			_waveDisplay = new WaveDisplay(displayContainer);
			
			_waveControls = new WaveControls(this);
			asset.addChild(_waveControls);
			
			_playLocatorBar = new PlayLocatorBar(this, 10, 154);
			_playLocatorBar.addEventListener(BasicLocatorBar.START_LOCATOR_CHANGE, playLocatorEventHandler);
			_playLocatorBar.addEventListener(BasicLocatorBar.END_LOCATOR_CHANGE, playLocatorEventHandler);
			
			_selectionLabel = new BodyText(11, Colors.WHITE, 200);
			_selectionLabel.autoSize = TextFieldAutoSize.LEFT;
			_selectionLabel.x = 257;
			_selectionLabel.y = 132;
			asset.addChild(_selectionLabel);
			
			_reverseButton = new TextButton(asset, 260, 85, "reverse", reverseClickHandler);
			_reverseButton.toggle = true;
			
			_transposeSlider = new BasicSlider(Slider.HORIZONTAL, asset, 10, 116, transposeSliderChangeHandler);
			_transposeSlider.setSliderParams(-12, 12, 0);
			_transposeSlider.setSize(210, 16);
			_transposeSlider.tick = 1;
			
			_transposeLabel = new BodyText(11, Colors.WHITE, 85);
			_transposeLabel.align = TextFormatAlign.RIGHT;
			_transposeLabel.autoSize = TextFieldAutoSize.RIGHT;
			_transposeLabel.x = 218;
			_transposeLabel.y = 132;
			asset.addChild(_transposeLabel);
			
			_tuneSlider = new BasicSlider(Slider.HORIZONTAL, asset, 10, 85, tuneSliderChangeHandler);
			_tuneSlider.setSliderParams(-0.5, 0.5, 0);
			_tuneSlider.setSize(210, 10);
			
			_tuneLabel = new BodyText(11, Colors.WHITE, 85);
			_tuneLabel.align = TextFormatAlign.RIGHT;
			_tuneLabel.autoSize = TextFieldAutoSize.RIGHT;
			_tuneLabel.x = 218;
			_tuneLabel.y = 97;
			asset.addChild(_tuneLabel);
			
			_activityLED = asset.tActivityLED;
		}

		/**
		 * Update locator position.
		 */
		private function updateLocators() : void
		{
			if(_startSample > -1 && _endSample > -1)
			{
				_playLocatorBar.startValue = (_startSample - _firstSampleInView) / _numSamplesInView;
				_playLocatorBar.endValue = (_endSample - _firstSampleInView) / _numSamplesInView;
				
				_selectionLabel.text = "selection start: " + _startSample + "  end: " + _endSample;
			}
		}

		/**
		 * 
		 */
		private function playLocatorEventHandler(event : Event) : void
		{
			switch(event.type)
			{
				case BasicLocatorBar.START_LOCATOR_CHANGE:
					dispatchEvent(new Event(WaveEditor.START_PLAY_LOCATOR_CHANGE));
					break;
				case BasicLocatorBar.END_LOCATOR_CHANGE:
					dispatchEvent(new Event(WaveEditor.END_PLAY_LOCATOR_CHANGE));
					break;
			}
		}

		/**
		 * 
		 */
		private function reverseClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(new Event(WaveEditor.REVERSE_CHANGE));
		}

		/**
		 * 
		 */
		private function transposeSliderChangeHandler(event : Event) : void
		{
			dispatchEvent(new Event(WaveEditor.TRANSPOSE_CHANGE));
		}

		/**
		 * 
		 */
		private function tuneSliderChangeHandler(event : Event) : void
		{
			dispatchEvent(new Event(WaveEditor.TUNE_CHANGE));
		}
	}
}
