package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.PushButton;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.text.BodyText;

	import flash.display.DisplayObjectContainer;
	import flash.display.Shape;
	import flash.events.MouseEvent;
	import flash.text.TextFormatAlign;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 17, 2010
	 */
	public class TextButton extends PushButton 
	{
		protected var _textLabel : BodyText;
		protected var _labelString : String = "";
		protected var _toggleIcon : Shape;

		public function TextButton(parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, label : String = "", defaultHandler : Function = null)
		{
			_labelString = label;
			
			super(parent, xpos, ypos, label, defaultHandler);
		}

		/**
		 * Draws the visual ui of the component.
		 */
		override public function draw() : void
		{
			_textLabel.width = _width;
			_textLabel.y = 1;
			
			_toggleIcon.graphics.clear();
			
			graphics.clear();
			graphics.beginFill(Colors.RUBY_RED);
			
			if(_enabled)
			{
				if(_over)
				{
					graphics.beginFill(Colors.BLOOD_RED);
				}
				else
				{
				}
				
				if(_down)
				{
					graphics.beginFill(Colors.BLOOD_RED);
					_textLabel.y = 2;
				}
			}
			else /** Disabled. */
			{
				graphics.beginFill(Colors.BLOOD_RED);
			}
			
			if(_selected)
			{
				_toggleIcon.graphics.beginFill(0xFFFFFF);
				_toggleIcon.graphics.drawRect(7, 7, 6, 6);
			}
			else
			{
			}
			
			graphics.drawRect(0, 0, _width, _height);
		}

		/**
		 * 
		 */
		override public function set selected(value : Boolean) : void
		{
			if(!_toggle) value = false;
			
			_selected = value;
			_down = _selected;
		}

		/**
		 * Initializes the component.
		 */
		override protected function init() : void
		{
			super.init();
			setSize(100, 20);
			
			mouseChildren = false;
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_textLabel = new BodyText(11, Colors.WHITE, _width);
			_textLabel.align = TextFormatAlign.CENTER;
			_textLabel.text = _labelString;
			addChild(_textLabel);
			
			_toggleIcon = new Shape();
			addChild(_toggleIcon);
			
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			addEventListener(MouseEvent.ROLL_OVER, onMouseOver);
		}

		/**
		 * Internal mouseOver handler.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onMouseOver(event : MouseEvent) : void
		{
			_over = true;
			invalidate();
			addEventListener(MouseEvent.ROLL_OUT, onMouseOut);
		}

		/**
		 * Internal mouseOut handler.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onMouseOut(event : MouseEvent) : void
		{
			_over = false;
			invalidate();
		}

		/**
		 * Internal mouseOut handler.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onMouseDown(event : MouseEvent) : void
		{
			_down = true;
			invalidate();
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}

		/**
		 * Internal mouseUp handler.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onMouseUp(event : MouseEvent) : void
		{
			if(_toggle && _over) _selected = !_selected;
			
			_down = _selected;
			
			invalidate();
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
	}
}
